﻿using System;
using System.Collections.Generic;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace LD22 {

   class SceneManager : DrawableGameComponent {

      public List<SceneLayer> Layers = new List<SceneLayer>();

      public SpriteBatch spriteBatch;

      public SceneManager(GameLD22 Game) : base(Game) {
         
      }

      protected override void LoadContent() {
         spriteBatch = new SpriteBatch(GraphicsDevice);
      }

      protected override void UnloadContent() {
         spriteBatch.Dispose();
      }

      public override void Update(GameTime gameTime) {
         var layers = this.Layers.ToArray();
         float elapsed = (float)Main.Game.LastElapsedTime;
         bool InputAvailable = true;
         for (int i = layers.Length - 1; i >= 0; i--) {
            if (layers[i].Manager != this) layers[i].Manager = this;
            if (!layers[i].Visible) continue;
            layers[i].HasInput = layers[i].GrabInput & InputAvailable;
            if (layers[i].GrabInput & InputAvailable) InputAvailable = false;
            layers[i].Update(elapsed);
         }
      }

      public override void Draw(GameTime gameTime) {
         //spriteBatch.Begin(SpriteSortMode.Immediate, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
         for (int i = 0; i < Layers.Count; i++) {
            Layers[i].Draw(spriteBatch);
         }
         //spriteBatch.End();
      }

   }

   class SceneLayer {

      public SceneManager Manager;

      public bool Visible = true;
      public bool GrabInput;

      public bool HasInput;

      public virtual void Draw(SpriteBatch batch) {

      }
      public virtual void Update(float ElapsedTime) {

      }

   }

   class LevelSceneLayer : SceneLayer {

      public LevelSceneLayer() {
         GrabInput = true;
      }

      public override void Draw(SpriteBatch batch) {
         Main.Game.Level.Draw(batch);
      }

      public override void Update(float ElapsedTime) {
         Main.Game.Level.Update(ElapsedTime, HasInput);
      }

   }

   class HUDLayer : SceneLayer {

      public HUDLayer() {
         GrabInput = false;
      }

      public override void Draw(SpriteBatch batch) {
         int health = Main.Game.Level.PlayerHealth;
         batch.Begin(  SpriteSortMode.Deferred, BlendState.AlphaBlend, SamplerState.PointClamp, DepthStencilState.Default, RasterizerState.CullNone);
         for (int i = 1; i <= Main.Game.Level.PlayerMaxHealth; i++) {
            if (health >= i) batch.Draw(ContentCache.UI_Heart, new Rectangle(i * 32, 32, 32, 32), Color.White);
            else batch.Draw(ContentCache.UI_HeartEmpty, new Rectangle(i * 32, 32, 32, 32), Color.White);
         }
         batch.End();
#if DEBUG
         batch.Begin();
         batch.DrawString(Main.Game.Font, "FPS: " + Main.Game.FPS, new Vector2(10, 10), Color.White);
         batch.End();
#endif
      }

      public override void Update(float ElapsedTime) {

      }

   }

   class PlayAgainLayer : SceneLayer {

      public string Message;
      public string AgainMsg;
      public string YesMsg;
      public string NoMsg;
      public Color Color;

      public float timer = 5f;

      public PlayAgainLayer(string message, string againMsg, string yesMsg, string noMsg, Color color, float delay) {
         GrabInput = true;
         this.Message = message;
         this.Color = color;
         this.timer = delay;
         this.AgainMsg = againMsg;
         this.YesMsg = yesMsg;
         this.NoMsg = noMsg;
      }

      public override void Draw(SpriteBatch batch) {
         if (timer > 0) return;
         float Y = Main.Game.ScreenHeight * 0.5f;
         batch.Begin();
         Functions.DrawStringCenter(batch, Main.Game.Font, Message, Y, Color);
         Functions.DrawStringCenter(batch, Main.Game.Font, AgainMsg, Y + 64, Color.White);
         Functions.DrawStringCenter(batch, Main.Game.Font, YesMsg +" (Enter, or A on a Gamepad) ", Y + 96, Color.White);
         Functions.DrawStringCenter(batch, Main.Game.Font, NoMsg + " (ESC, or Back on a Gamepad)", Y + 128, Color.White);
         batch.End();
      }

      public override void Update(float ElapsedTime) {
         if (timer > 0) {
            timer -= ElapsedTime;
            return;
         }
         if (InputHandler.KeyPressed(Keys.Enter) || InputHandler.ButtonPressed(Buttons.A)) {
            Main.Game.Scene.Layers.Remove(this);
            Main.Game.Level.Load("level1");
         } else if (InputHandler.KeyPressed(Keys.Escape) || InputHandler.ButtonPressed(Buttons.Back)) {
            Main.Game.Exit();
         }
      }

   }
}
